/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdio.h>
#include <string.h>

#include <aul.h>
#include <aul_comp_context.h>

#include "component_context.h"
#include "component_context_internal.h"
#include "component_manager_private.h"

#define HANDLE_TO_AUL_HANDLE(h) ((aul_comp_context_h)(component_context_h)(h))
#define AUL_HANDLE_TO_HANDLE(h) ((component_context_h)(aul_comp_context_h)(h))

static int __convert_aul_error(int ret)
{
	switch (ret) {
	case AUL_R_ENOMEM:
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	case AUL_R_ENOENT:
	case AUL_R_ENOAPP:
		return COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT;
	case AUL_R_EILLACC:
		return COMPONENT_MANAGER_ERROR_PERMISSION_DENIED;
	case AUL_R_EINVAL:
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	default:
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}
}

int component_context_create(const char *comp_id, component_context_h *handle)
{
	aul_comp_context_h context = NULL;
	int ret;

	if (!comp_id || !handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_create(comp_id, &context);
	if (ret != AUL_R_OK) {
		LOGE("Failed to create running component context");
		return __convert_aul_error(ret);
	}

	*handle = AUL_HANDLE_TO_HANDLE(context);

	return COMPONENT_MANAGER_ERROR_NONE;
}

int component_context_is_running(component_context_h handle, bool *running)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	bool value = false;
	int ret;

	if (!handle || !running) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_is_running(context, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to check running");
		return __convert_aul_error(ret);
	}

	*running = value;

	return COMPONENT_MANAGER_ERROR_NONE;
}

int component_context_resume(component_context_h handle)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	int ret;

	if (!handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_resume(context);
	if (ret != AUL_R_OK) {
		LOGE("Failed to send a resume request. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

int component_context_pause(component_context_h handle)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	int ret;

	if (!handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_pause(context);
	if (ret != AUL_R_OK) {
		LOGE("Failed to send a pause request. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

int component_context_terminate_bg_component(component_context_h handle)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	int ret;

	if (!handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_terminate_bg_comp(context);
	if (ret != AUL_R_OK) {
		LOGE("Failed to send a terminate request. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

int component_context_terminate(component_context_h handle)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	int ret;

	if (!handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_terminate(context);
	if (ret != AUL_R_OK) {
		LOGE("Failed to send a terminate request. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

int component_context_foreach(component_manager_component_context_cb callback,
		void *user_data)
{
	int ret;

	if (!callback) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_foreach_comp_context(
			(aul_comp_context_cb)callback, user_data);
	if (ret != AUL_R_OK) {
		LOGE("Failed to retrieve component context. error(%d)", ret);
		return __convert_aul_error(ret);
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_context_destroy(component_context_h handle)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);

	if (!handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	aul_comp_context_destroy(context);

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_context_get_app_id(component_context_h handle,
		char **app_id)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !app_id) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_get_app_id(context, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get application ID");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	*app_id = strdup(value);
	if (*app_id == NULL) {
		LOGE("Failed to duplicate application ID");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_context_get_component_id(component_context_h handle,
		char **component_id)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !component_id) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_get_comp_id(context, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get component ID");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	*component_id = strdup(value);
	if (*component_id == NULL) {
		LOGE("Failed to duplicate component ID");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_context_get_instance_id(component_context_h handle,
		char **instance_id)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !instance_id) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_get_instance_id(context, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get instance ID");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	*instance_id = strdup(value);
	if (*instance_id == NULL) {
		LOGE("Failed to duplicate instance ID");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

static component_state_e __get_component_state(int status)
{
	component_state_e state;

	switch (status) {
	case COMP_STATUS_INITIALIZED:
		state = COMPONENT_STATE_INITIALIZED;
		break;
	case COMP_STATUS_CREATED:
		state = COMPONENT_STATE_CREATED;
		break;
	case COMP_STATUS_STARTED:
		state = COMPONENT_STATE_STARTED;
		break;
	case COMP_STATUS_RESUMED:
		state = COMPONENT_STATE_RESUMED;
		break;
	case COMP_STATUS_PAUSED:
		state = COMPONENT_STATE_PAUSED;
		break;
	default:
		state = COMPONENT_STATE_DESTROYED;
		break;
	}

	return state;
}

EXPORT_API int component_context_get_component_state(component_context_h handle,
		component_state_e *state)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	int value = -1;
	int ret;

	if (!handle || !state) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_get_status(context, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get status");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	*state = __get_component_state(value);

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_context_is_terminated(component_context_h handle,
		bool *terminated)
{
	bool value = false;
	int ret;

	if (!handle || !terminated) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = component_context_is_running(handle, &value);
	if (ret != COMPONENT_MANAGER_ERROR_NONE)
		return ret;

	*terminated = value ? false : true;

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_context_is_subcomponent(component_context_h handle,
		bool *is_subcomponent)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	bool value = false;
	int ret;

	if (!handle || !is_subcomponent) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_is_sub_comp(context, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to check sub comp");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	*is_subcomponent = value;

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_context_clone(component_context_h *clone,
		component_context_h handle)
{
	aul_comp_context_h context = HANDLE_TO_AUL_HANDLE(handle);
	aul_comp_context_h new_context = NULL;
	int ret;

	if (!clone || !handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_context_clone(context, &new_context);
	if (ret != AUL_R_OK) {
		LOGE("Failed to clone component context");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	*clone = AUL_HANDLE_TO_HANDLE(new_context);

	return COMPONENT_MANAGER_ERROR_NONE;
}
