/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef __TIZEN_APPFW_COMPONENT_CONTEXT_INTERNAL_H
#define __TIZEN_APPFW_COMPONENT_CONTEXT_INTERNAL_H

#include <component_manager.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @file component_context_internal.h
 */

/**
 * @addtogroup CAPI_COMPONENT_CONTEXT_MODULE
 * @{
 */

/**
 * @brief Creates the component context handle.
 * @since_tizen 5.5
 * @remarks You MUST release @c handle using component_context_destroy().
 * @privlevel public
 * @privilege %http://tizen.org/privilege/packagemanager.info
 *
 * @param[in]   comp_id         The component ID
 * @param[out]  handle          The component context handle
 * @return      @c 0 on success,
 *              otherwise a negative error value
 * @retval #COMPONENT_MANAGER_ERROR_NONE Successful
 * @retval #COMPONENT_MANAGER_ERROR_PERMISSION_DENIED Permission denied
 * @retval #COMPONENT_MANAGER_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY Out of memory
 * @retval #COMPONENT_MANAGER_ERROR_IO_ERROR I/O error
 * @retval #COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT No such component
 */
int component_context_create(const char *comp_id, component_context_h *handle);

/**
 * @brief Checks whether the component is running or not.
 * @since_tizen 5.5
 *
 * @param[in]   handle          The component context handle
 * @param[out]  running         @c true if the component if running, \n
 *                              otherwise @c false if not running
 * @return      @c 0 on success,
 *              otherwise a negative error value
 * @retval #COMPONENT_MANAGER_ERROR_NONE Successful
 * @retval #COMPONENT_MANAGER_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #COMPONENT_MANAGER_ERROR_IO_ERROR I/O error
 */
int component_context_is_running(component_context_h handle, bool *running);

/**
 * @brief Sends the resume request.
 * @since_tizen 5.5
 * @privlevel public
 * @privilege %http://tizen.org/privilege/appmanager.launch
 *
 * @param[in]   handle          The component context handle
 * @return      @c 0 on success,
 *              otherwise a negative error value
 * @retval #COMPONENT_MANAGER_ERROR_NONE Successful
 * @retval #COMPONENT_MANAGER_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #COMPONENT_MANAGER_ERROR_IO_ERROR I/O error
 * @retval #COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT No such component
 * @retval #COMPONENT_MANAGER_ERROR_PERMISSION_DENIED Permission denied
 */
int component_context_resume(component_context_h handle);

/**
 * @brief Sends the resume request.
 * @since_tizen 5.5
 * @privlevel public
 * @privilege %http://tizen.org/privilege/appmanager.launch
 *
 * @param[in]   handle          The component context handle
 * @return      @c 0 on success,
 *              otherwise a negative error value
 * @retval #COMPONENT_MANAGER_ERROR_NONE Successful
 * @retval #COMPONENT_MANAGER_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #COMPONENT_MANAGER_ERROR_IO_ERROR I/O error
 * @retval #COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT No such component
 * @retval #COMPONENT_MANAGER_ERROR_PERMISSION_DENIED Permission denied
 */
int component_context_pause(component_context_h handle);

/**
 * @brief Sends the request for teminating the background component.
 * @since_tizen 5.5
 * @privlevel public
 * @privilege %http://tizen.org/privilege/appmanager.bg.kill
 *
 * @param[in]   handle          The component context handle
 * @return      @c 0 on success,
 *              otherwise a negative error value
 * @retval #COMPONENT_MANAGER_ERROR_NONE Successful
 * @retval #COMPONENT_MANAGER_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #COMPONENT_MANAGER_ERROR_IO_ERROR I/O error
 * @retval #COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT No such component
 * @retval #COMPONENT_MANAGER_ERROR_PERMISSION_DENIED Permission denied
 */
int component_context_terminate_bg_component(component_context_h handle);

/**
 * @brief Sends the request for teminating the running component.
 * @since_tizen 5.5
 * @privlevel platform
 * @privilege %http://tizen.org/privilege/appmanager.kill
 *
 * @param[in]   handle          The component context handle
 * @return      @c 0 on success,
 *              otherwise a negative error value
 * @retval #COMPONENT_MANAGER_ERROR_NONE Successful
 * @retval #COMPONENT_MANAGER_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #COMPONENT_MANAGER_ERROR_IO_ERROR I/O error
 * @retval #COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT No such component
 * @retval #COMPONENT_MANAGER_ERROR_PERMISSION_DENIED Permission denied
 */
int component_context_terminate(component_context_h handle);

/**
 * @brief Retrieves all running components context.
 * @since_tizen 5.5
 * @privlevel public
 * @privilege %http://tizen.org/privilege/packagemanager.info
 *
 * @param[in]   callback        The callback function to invoke
 * @param[in]   user_data       The user data to be passed from the callback function
 * @return      @c 0 on success,
 *              otherwise a negative error value
 * @retval #COMPONENT_MANAGER_ERROR_NONE Successful
 * @retval #COMPONENT_MANAGER_ERROR_PERMISSION_DENIED Permission denied
 * @retval #COMPONENT_MANAGER_ERROR_INVALID_PARAMETER Invalid parameter
 * @retval #COMPONENT_MANAGER_ERROR_IO_ERROR I/O error
 * @retval #COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT No such component
 */
int component_context_foreach(component_manager_component_context_cb callback, void *user_data);

/**
 * @}
 */

#ifdef __cplusplus
}
#endif

#endif /* __TIZEN_APPFW_COMPONENT_CONTEXT_INTERNAL_H */
