/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include <aul.h>
#include <aul_comp_info.h>

#include "component_info.h"
#include "component_info_internal.h"
#include "component_manager.h"
#include "component_manager_private.h"

#define HANDLE_TO_AUL_HANDLE(h) ((aul_comp_info_h)(component_info_h)(h))
#define AUL_HANDLE_TO_HANDLE(h) ((component_info_h)(aul_comp_info_h)(h))

int component_info_foreach(component_manager_component_info_cb callback,
		void *user_data)
{
	int ret;

	if (!callback) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_foreach_comp_info((aul_comp_info_cb)callback,
			user_data);
	if (ret != AUL_R_OK) {
		LOGE("Failed to retrieve component info. error(%d)", ret);
		if (ret == AUL_R_EILLACC)
			return COMPONENT_MANAGER_ERROR_PERMISSION_DENIED;
		if (ret == AUL_R_ENOENT)
			return COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT;
		if (ret == AUL_R_ENOMEM)
			return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;

		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_create(const char *comp_id,
		component_info_h *handle)
{
	aul_comp_info_h h = NULL;
	int ret;

	if (!comp_id || !handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_create(comp_id, &h);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get component info. error(%d)", ret);
		if (ret == AUL_R_EILLACC)
			return COMPONENT_MANAGER_ERROR_PERMISSION_DENIED;
		if (ret == AUL_R_ENOENT)
			return COMPONENT_MANAGER_ERROR_NO_SUCH_COMPONENT;

		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	*handle = AUL_HANDLE_TO_HANDLE(h);

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_destroy(component_info_h handle)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);

	if (!handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	aul_comp_info_destroy(h);

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_get_app_id(component_info_h handle,
		char **app_id)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !app_id) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_get_app_id(h, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get application ID");
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	*app_id = strdup(value);
	if (*app_id == NULL) {
		LOGE("Failed to duplicate application ID");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_get_component_id(component_info_h handle,
		char **component_id)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !component_id) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_get_comp_id(h, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get component ID");
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	*component_id = strdup(value);
	if (*component_id == NULL) {
		LOGE("Failed to duplicate component ID");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_get_component_type(component_info_h handle,
		component_info_component_type_e *type)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !type) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_get_type(h, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get component type");
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	if (value && !strcmp(value, "frame"))
		*type = COMPONENT_INFO_COMPONENT_TYPE_FRAME;
	else if (value && !strcmp(value, "service"))
		*type = COMPONENT_INFO_COMPONENT_TYPE_SERVICE;
	else
		return COMPONENT_MANAGER_ERROR_IO_ERROR;

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_is_icon_display(component_info_h handle,
		bool *icon_display)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	bool value = false;
	int ret;

	if (!handle || !icon_display) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_is_icon_display(h, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to check icon display");
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	*icon_display = value;

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_is_managed_by_task_manager(
		component_info_h handle, bool *managed)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	bool value = false;
	int ret;

	if (!handle || !managed) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_is_taskmanage(h, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to check taskmanage");
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	*managed = value;

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_get_icon(component_info_h handle, char **path)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !path) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_get_icon(h, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get component icon");
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	*path = strdup(value);
	if (*path == NULL) {
		LOGE("Failed to duplicate component icon");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_get_label(component_info_h handle, char **label)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !label) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_get_label(h, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get component label");
		return COMPONENT_MANAGER_ERROR_IO_ERROR;
	}

	*label = strdup(value);
	if (*label == NULL) {
		LOGE("Failed to duplicate component label");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_get_localized_label(component_info_h handle,
		const char *locale, char **label)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	const char *value = NULL;
	int ret;

	if (!handle || !locale || !label) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_get_localed_label(h, locale, &value);
	if (ret != AUL_R_OK) {
		LOGE("Failed to get localed label");
		if (ret == AUL_R_ENOENT)
			return COMPONENT_MANAGER_ERROR_LABEL_NOT_FOUND;

		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	*label = strdup(value);
	if (*label == NULL) {
		LOGE("Failed to duplicate localized label");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	return COMPONENT_MANAGER_ERROR_NONE;
}

EXPORT_API int component_info_clone(component_info_h *clone,
		component_info_h handle)
{
	aul_comp_info_h h = HANDLE_TO_AUL_HANDLE(handle);
	aul_comp_info_h dup_h = NULL;
	int ret;

	if (!clone || !handle) {
		LOGE("Invalid parameter");
		return COMPONENT_MANAGER_ERROR_INVALID_PARAMETER;
	}

	ret = aul_comp_info_clone(h, &dup_h);
	if (ret != AUL_R_OK) {
		LOGE("Failed to clone component info");
		return COMPONENT_MANAGER_ERROR_OUT_OF_MEMORY;
	}

	*clone = AUL_HANDLE_TO_HANDLE(dup_h);

	return COMPONENT_MANAGER_ERROR_NONE;
}
