/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <aul.h>
#include <gtest/gtest.h>

#include <iostream>
#include <memory>

#include "mock_aul_comp_types.h"
#include "mock_aul_comp_context.h"
#include "component_manager.h"

static int __fake_aul_comp_context_create(const char* comp_id,
    aul_comp_context_h* handle) {
  char* h = static_cast<char*>(malloc(sizeof(char)));
  if (!h)
    return AUL_R_ENOMEM;

  *handle = static_cast<aul_comp_context_h>(h);

  return AUL_R_OK;
}

static int __fake_aul_comp_context_destroy(aul_comp_context_h handle) {
  if (!handle)
    return AUL_R_EINVAL;

  free(static_cast<void*>(handle));

  return AUL_R_OK;
}

static int __fake_aul_comp_context_get_app_id(aul_comp_context_h handle,
    const char** app_id) {
  *app_id = "org.tizen.helloworld";
  return AUL_R_OK;
}

static int __fake_aul_comp_context_get_instance_id(aul_comp_context_h handle,
    const char** instance_id) {
  *instance_id = "2019@TestComponent";
  return AUL_R_OK;
}

static int __fake_aul_comp_context_get_comp_id(aul_comp_context_h handle,
    const char** comp_id) {
  *comp_id = "TestComponent";
  return AUL_R_OK;
}

static int __fake_aul_comp_context_get_type(aul_comp_context_h handle,
    const char** type) {
  *type = "frame";
  return AUL_R_OK;
}

static int __fake_aul_comp_context_get_pid(aul_comp_context_h handle,
    pid_t* pid) {
  *pid = getpid();
  return AUL_R_OK;
}

static int __fake_aul_comp_context_get_status(aul_comp_context_h handle,
    int* status) {
  *status = COMP_STATUS_RESUMED;
  return AUL_R_OK;
}

static int __fake_aul_comp_context_is_sub_comp(aul_comp_context_h handle,
    bool* is_sub_comp) {
  *is_sub_comp = false;
  return AUL_R_OK;
}

static int __fake_aul_comp_context_clone(aul_comp_context_h handle,
    aul_comp_context_h *clone) {
  char* h = static_cast<char*>(malloc(sizeof(char)));
  if (!h)
    return AUL_R_ENOMEM;

  *clone = static_cast<aul_comp_context_h>(h);

  return AUL_R_OK;
}

static int __fake_aul_comp_context_is_running(aul_comp_context_h handle,
    bool *running) {
  *running = true;
  return AUL_R_OK;
}

static int __fake_aul_comp_context_resume(aul_comp_context_h handle) {
  return AUL_R_OK;
}

static int __fake_aul_comp_context_pause(aul_comp_context_h handle) {
  return AUL_R_OK;
}

static int __fake_aul_comp_context_terminate_bg_comp(
    aul_comp_context_h handle) {
  return AUL_R_OK;
}

static int __fake_aul_comp_context_terminate(aul_comp_context_h handle) {
  return AUL_R_OK;
}

class ComponentContextTest : public testing::Test {
 public:
  virtual void SetUp() {
    aul_comp_context_create_fake.custom_fake =
      __fake_aul_comp_context_create;
    aul_comp_context_destroy_fake.custom_fake =
      __fake_aul_comp_context_destroy;
    aul_comp_context_get_app_id_fake.custom_fake =
      __fake_aul_comp_context_get_app_id;
    aul_comp_context_get_instance_id_fake.custom_fake =
      __fake_aul_comp_context_get_instance_id;
    aul_comp_context_get_comp_id_fake.custom_fake =
      __fake_aul_comp_context_get_comp_id;
    aul_comp_context_get_type_fake.custom_fake =
      __fake_aul_comp_context_get_type;
    aul_comp_context_get_pid_fake.custom_fake =
      __fake_aul_comp_context_get_pid;
    aul_comp_context_get_status_fake.custom_fake =
      __fake_aul_comp_context_get_status;
    aul_comp_context_is_sub_comp_fake.custom_fake =
      __fake_aul_comp_context_is_sub_comp;
    aul_comp_context_clone_fake.custom_fake =
      __fake_aul_comp_context_clone;
    aul_comp_context_is_running_fake.custom_fake =
      __fake_aul_comp_context_is_running;
    aul_comp_context_resume_fake.custom_fake =
      __fake_aul_comp_context_resume;
    aul_comp_context_pause_fake.custom_fake =
      __fake_aul_comp_context_pause;
    aul_comp_context_terminate_bg_comp_fake.custom_fake =
      __fake_aul_comp_context_terminate_bg_comp;
    aul_comp_context_terminate_fake.custom_fake =
      __fake_aul_comp_context_terminate;

    component_manager_get_component_context("TestComponent", &handle_);
  }

  virtual void TearDown() {
    if (handle_ != nullptr) {
      component_context_destroy(handle_);
      handle_ = nullptr;
    }
  }

  component_context_h GetHandle() const {
    return handle_;
  }

  void SetHandle(component_context_h handle) {
    handle_ = handle;
  }

 private:
  component_context_h handle_ = nullptr;
};

TEST_F(ComponentContextTest, component_context_destroy_Positive) {
  component_context_h handle = GetHandle();

  int ret = component_context_destroy(handle);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  SetHandle(nullptr);
}

TEST_F(ComponentContextTest, component_context_destroy_Negative) {
  int ret = component_context_destroy(nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentContextTest, component_context_get_app_id_Positive) {
  component_context_h handle = GetHandle();
  char* app_id = nullptr;

  int ret = component_context_get_app_id(handle, &app_id);
  auto p = std::unique_ptr<char, decltype(std::free)*>(app_id, std::free);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(std::string(p.get()), "org.tizen.helloworld");
}

TEST_F(ComponentContextTest, component_context_get_app_id_Negative) {
  int ret = component_context_get_app_id(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentContextTest, component_context_get_component_id_Positive) {
  component_context_h handle = GetHandle();
  char* component_id = nullptr;

  int ret = component_context_get_component_id(handle, &component_id);
  auto p = std::unique_ptr<char, decltype(std::free)*>(component_id, std::free);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(std::string(p.get()), "TestComponent");
}

TEST_F(ComponentContextTest, component_context_get_component_id_Negative) {
  int ret = component_context_get_component_id(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentContextTest, component_context_get_instance_id_Positive) {
  component_context_h handle = GetHandle();
  char* instance_id = nullptr;

  int ret = component_context_get_instance_id(handle, &instance_id);
  auto p = std::unique_ptr<char, decltype(std::free)*>(instance_id, std::free);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(std::string(p.get()), "2019@TestComponent");
}

TEST_F(ComponentContextTest, component_context_get_instance_id_Negative) {
  int ret = component_context_get_instance_id(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentContextTest, component_context_get_component_state_Positive) {
  component_context_h handle = GetHandle();
  component_state_e state = COMPONENT_STATE_INITIALIZED;

  int ret = component_context_get_component_state(handle, &state);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(state, COMPONENT_STATE_RESUMED);
}

TEST_F(ComponentContextTest, component_context_get_component_state_Negative) {
  int ret = component_context_get_component_state(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentContextTest, component_context_is_terminated_Positive) {
  component_context_h handle = GetHandle();
  bool terminated;

  int ret = component_context_is_terminated(handle, &terminated);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(terminated, false);
}

TEST_F(ComponentContextTest, component_context_is_terminated_Negative) {
  int ret = component_context_is_terminated(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentContextTest, component_context_is_subcomponent_Positive) {
  component_context_h handle = GetHandle();
  bool is_subcomponent;

  int ret = component_context_is_subcomponent(handle, &is_subcomponent);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(is_subcomponent, false);
}

TEST_F(ComponentContextTest, component_context_is_subcomponent_Negative) {
  int ret = component_context_is_subcomponent(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentContextTest, component_context_clone_Positive) {
  component_context_h handle = GetHandle();
  component_context_h clone = nullptr;

  int ret = component_context_clone(&clone, handle);
  auto p = std::unique_ptr<std::remove_pointer<component_context_h>::type,
       decltype(component_context_destroy)*>(clone, component_context_destroy);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_NE(p, nullptr);
  EXPECT_NE(p.get(), nullptr);
}

TEST_F(ComponentContextTest, component_context_clone_Negative) {
  int ret = component_context_clone(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}
