/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <aul.h>
#include <gtest/gtest.h>

#include <iostream>
#include <memory>

#include "mock_aul_comp_info.h"
#include "component_manager.h"

static const char ICON_PATH[] =
  "/opt/usr/home/owner/apps_rw/org.tizen.helloworld/shared/res/helloworld.png";

static int __fake_aul_comp_info_create(const char* compid,
    aul_comp_info_h* handle) {
  char* h = static_cast<char*>(malloc(sizeof(char)));
  if (!h)
    return AUL_R_ENOMEM;

  *handle = static_cast<aul_comp_info_h>(h);

  return AUL_R_OK;
}

static int __fake_aul_comp_info_destroy(
    aul_comp_info_h handle) {
  if (!handle)
    return AUL_R_EINVAL;

  free(static_cast<void*>(handle));

  return AUL_R_OK;
}

static int __fake_aul_comp_info_get_app_id(aul_comp_info_h handle,
    const char** appid) {
  *appid = "org.tizen.helloworld";
  return AUL_R_OK;
}

static int __fake_aul_comp_info_get_comp_id(aul_comp_info_h handle,
    const char** compid) {
  *compid = "TestComponent";
  return AUL_R_OK;
}

static int __fake_aul_comp_info_get_type(aul_comp_info_h handle,
    const char** type) {
  *type = "frame";
  return AUL_R_OK;
}

static int __fake_aul_comp_info_is_icon_display(aul_comp_info_h handle,
    bool* nodisplay) {
  *nodisplay = true;
  return AUL_R_OK;
}

static int __fake_aul_comp_info_is_taskmanage(
    aul_comp_info_h handle, bool* taskmanage) {
  *taskmanage = true;
  return AUL_R_OK;
}

static int __fake_aul_comp_info_get_icon(aul_comp_info_h handle,
    const char** icon) {
  *icon = ICON_PATH;
  return AUL_R_OK;
}

static int __fake_aul_comp_info_get_label(aul_comp_info_h handle,
    const char** label) {
  *label = "TestComponent";
  return AUL_R_OK;
}

static int __fake_aul_comp_info_get_localed_label(aul_comp_info_h handle,
    const char* locale, char** label) {
  *label = strdup("Kor_TestComponent");
  return AUL_R_OK;
}

static int __fake_aul_comp_info_clone(
    aul_comp_info_h handle, aul_comp_info_h* clone) {
  char* h = static_cast<char *>(malloc(sizeof(char)));
  if (!h)
    return AUL_R_ERROR;

  *clone = static_cast<aul_comp_info_h>(h);

  return AUL_R_OK;
}

class ComponentInfoTest : public testing::Test {
 public:
  virtual void SetUp() {
    aul_comp_info_create_fake.custom_fake =
        __fake_aul_comp_info_create;
    aul_comp_info_destroy_fake.custom_fake =
        __fake_aul_comp_info_destroy;
    aul_comp_info_get_app_id_fake.custom_fake =
        __fake_aul_comp_info_get_app_id;
    aul_comp_info_get_comp_id_fake.custom_fake =
        __fake_aul_comp_info_get_comp_id;
    aul_comp_info_get_type_fake.custom_fake =
        __fake_aul_comp_info_get_type;
    aul_comp_info_is_icon_display_fake.custom_fake =
        __fake_aul_comp_info_is_icon_display;
    aul_comp_info_is_taskmanage_fake.custom_fake =
        __fake_aul_comp_info_is_taskmanage;
    aul_comp_info_get_icon_fake.custom_fake =
        __fake_aul_comp_info_get_icon;
    aul_comp_info_get_label_fake.custom_fake =
        __fake_aul_comp_info_get_label;
    aul_comp_info_get_localed_label_fake.custom_fake =
        __fake_aul_comp_info_get_localed_label;
    aul_comp_info_clone_fake.custom_fake =
        __fake_aul_comp_info_clone;

    component_info_create("TestComponent", &handle_);
  }

  virtual void TearDown() {
    if (handle_ != nullptr) {
      component_info_destroy(handle_);
      handle_ = nullptr;
    }
  }

  component_info_h GetHandle() const {
    return handle_;
  }

  void SetHandle(component_info_h handle) {
    handle_ = handle;
  }

 private:
  component_info_h handle_ = nullptr;
};

TEST_F(ComponentInfoTest, component_info_create_Positive) {
  component_info_h handle = nullptr;
  int ret = component_info_create("TestComponent", &handle);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);

  // Post: Destroys the handle
  component_info_destroy(handle);
}

TEST_F(ComponentInfoTest, component_info_create_Negative) {
  int ret = component_info_create(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_destroy_Positive) {
  component_info_h handle = GetHandle();

  int ret = component_info_destroy(handle);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  SetHandle(nullptr);
}

TEST_F(ComponentInfoTest, component_info_destroy_Negative) {
  int ret = component_info_destroy(nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_get_app_id_Positive) {
  component_info_h handle = GetHandle();

  char* value = nullptr;
  int ret = component_info_get_app_id(handle, &value);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  auto p = std::unique_ptr<char, decltype(std::free)*>(value, std::free);
  EXPECT_EQ(std::string(p.get()), "org.tizen.helloworld");
}

TEST_F(ComponentInfoTest, component_info_get_app_id_Negative) {
  int ret = component_info_get_app_id(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_get_component_id_Positive) {
  component_info_h handle = GetHandle();

  char* value = nullptr;
  int ret = component_info_get_component_id(handle, &value);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  auto p = std::unique_ptr<char, decltype(std::free)*>(value, std::free);
  EXPECT_EQ(std::string(p.get()), "TestComponent");
}

TEST_F(ComponentInfoTest, component_info_get_component_id_Negative) {
  int ret = component_info_get_component_id(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_get_component_type_Positive) {
  component_info_h handle = GetHandle();

  component_info_component_type_e type;
  int ret = component_info_get_component_type(handle, &type);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(type, COMPONENT_INFO_COMPONENT_TYPE_FRAME);
}

TEST_F(ComponentInfoTest, component_info_get_component_type_Negative) {
  int ret = component_info_get_component_type(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_is_icon_display_Positive) {
  component_info_h handle = GetHandle();

  bool icon_display = false;
  int ret = component_info_is_icon_display(handle, &icon_display);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(icon_display, true);
}

TEST_F(ComponentInfoTest, component_info_is_icon_display_Negative) {
  int ret = component_info_is_icon_display(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_is_managed_by_task_manager_Positive) {
  component_info_h handle = GetHandle();

  bool managed = false;
  int ret = component_info_is_managed_by_task_manager(handle, &managed);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(managed, true);
}

TEST_F(ComponentInfoTest, component_info_is_managed_by_task_manager_Negative) {
  int ret = component_info_is_managed_by_task_manager(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_get_icon_Positive) {
  component_info_h handle = GetHandle();

  char* value = nullptr;
  int ret = component_info_get_icon(handle, &value);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  auto p = std::unique_ptr<char, decltype(std::free)*>(value, std::free);
  EXPECT_EQ(std::string(p.get()), ICON_PATH);
}

TEST_F(ComponentInfoTest, component_info_get_icon_Negative) {
  int ret = component_info_get_icon(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_get_label_Positive) {
  component_info_h handle = GetHandle();

  char* value = nullptr;
  int ret = component_info_get_label(handle, &value);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  auto p = std::unique_ptr<char, decltype(std::free)*>(value, std::free);
  EXPECT_EQ(std::string(p.get()), "TestComponent");
}

TEST_F(ComponentInfoTest, component_info_get_label_Nagative) {
  int ret = component_info_get_label(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_get_localized_label_Positive) {
  component_info_h handle = GetHandle();

  char* value = nullptr;
  int ret = component_info_get_localized_label(handle, "ko-kr", &value);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  auto p = std::unique_ptr<char, decltype(std::free)*>(value, std::free);
  EXPECT_EQ(std::string(p.get()), "Kor_TestComponent");
}

TEST_F(ComponentInfoTest, component_info_get_localized_label_Negative) {
  int ret = component_info_get_localized_label(nullptr, nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentInfoTest, component_info_clone_Positive) {
  component_info_h handle = GetHandle();

  component_info_h clone = nullptr;
  int ret = component_info_clone(&clone, handle);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  component_info_destroy(clone);
}

TEST_F(ComponentInfoTest, component_info_clone_Negative) {
  int ret = component_info_clone(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}
