/*
 * Copyright (c) 2019 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h>
#include <aul.h>
#include <gtest/gtest.h>

#include <iostream>
#include <memory>

#include "mock_aul_comp_info.h"
#include "mock_aul_comp_context.h"
#include "component_manager.h"

static int __fake_aul_comp_info_foreach_comp_info(aul_comp_info_cb callback,
    void* user_data) {
  aul_comp_info_h handle = nullptr;
  callback(handle, user_data);
  return AUL_R_OK;
}

static int __fake_aul_comp_info_create(const char* comp_id,
    aul_comp_info_h* handle) {
  char* h = static_cast<char*>(malloc(sizeof(char)));
  if (!h)
    return AUL_R_ENOMEM;

  *handle = static_cast<aul_comp_info_h>(h);
  return AUL_R_OK;
}

static int __fake_aul_comp_info_destroy(aul_comp_info_h handle) {
  if (!handle)
    return AUL_R_EINVAL;

  std::free(static_cast<void*>(handle));
  return AUL_R_OK;
}

static int __fake_aul_comp_context_foreach_comp_context(
    aul_comp_context_cb callback, void* user_data) {
  aul_comp_context_h handle = nullptr;
  callback(handle, user_data);
  return AUL_R_OK;
}

static int __fake_aul_comp_context_create(const char* comp_id,
    aul_comp_context_h* handle) {
  char* h = static_cast<char*>(malloc(sizeof(char)));
  if (!h)
    return AUL_R_ENOMEM;

  *handle = static_cast<aul_comp_context_h>(h);
  return AUL_R_OK;
}

static int __fake_aul_comp_context_destroy(aul_comp_context_h handle) {
  if (!handle)
    return AUL_R_EINVAL;

  std::free(static_cast<void*>(handle));
  return AUL_R_OK;
}

static int __fake_aul_comp_context_is_running(aul_comp_context_h handle,
    bool* running) {
  *running = true;
  return AUL_R_OK;
}

static int __fake_aul_comp_context_resume(aul_comp_context_h handle) {
  return AUL_R_OK;
}

static int __fake_aul_comp_context_terminate_bg_comp(
    aul_comp_context_h handle) {
  return AUL_R_OK;
}

class ComponentManagerTest : public testing::Test {
 public:
  virtual void SetUp() {
    aul_comp_info_foreach_comp_info_fake.custom_fake =
      __fake_aul_comp_info_foreach_comp_info;
    aul_comp_info_create_fake.custom_fake =
      __fake_aul_comp_info_create;
    aul_comp_info_destroy_fake.custom_fake =
      __fake_aul_comp_info_destroy;
    aul_comp_context_foreach_comp_context_fake.custom_fake =
      __fake_aul_comp_context_foreach_comp_context;
    aul_comp_context_create_fake.custom_fake =
      __fake_aul_comp_context_create;
    aul_comp_context_destroy_fake.custom_fake =
      __fake_aul_comp_context_destroy;
    aul_comp_context_is_running_fake.custom_fake =
      __fake_aul_comp_context_is_running;
    aul_comp_context_resume_fake.custom_fake =
      __fake_aul_comp_context_resume;
    aul_comp_context_terminate_bg_comp_fake.custom_fake =
      __fake_aul_comp_context_terminate_bg_comp;

    component_manager_get_component_context("TestComponent", &handle_);
    component_context_cb_touched_ = false;
    component_info_cb_touched_ = false;
  }

  virtual void TearDown() {
    if (handle_ != nullptr) {
      component_context_destroy(handle_);
      handle_ = nullptr;
    }
  }

  component_context_h GetHandle() const {
    return handle_;
  }

  void SetHandle(component_context_h handle) {
    handle_ = handle;
  }

 public:
  bool component_context_cb_touched_ = false;
  bool component_info_cb_touched_ = false;

 private:
  component_context_h handle_ = nullptr;
};

TEST_F(ComponentManagerTest,
    component_manager_foreach_component_context_Positive) {
  int ret = component_manager_foreach_component_context(
      [](component_context_h handle, void* user_data) -> bool {
        ComponentManagerTest* p = static_cast<ComponentManagerTest*>(user_data);
        p->component_context_cb_touched_ = true;
        return true;
      }, this);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(component_context_cb_touched_, true);
}

TEST_F(ComponentManagerTest,
    comopnent_manager_foreach_component_context_Negative) {
  int ret = component_manager_foreach_component_context(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentManagerTest,
    component_manager_foreach_component_info_Positive) {
  int ret = component_manager_foreach_component_info(
      [](component_info_h handle, void* user_data) -> bool {
        ComponentManagerTest* p = static_cast<ComponentManagerTest*>(user_data);
        p->component_info_cb_touched_ = true;
        return true;
      }, this);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(component_info_cb_touched_, true);
}

TEST_F(ComponentManagerTest,
    component_manager_foreach_component_info_Negative) {
  int ret = component_manager_foreach_component_info(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentManagerTest, component_manager_get_component_context_Positive) {
  component_context_h handle = nullptr;
  int ret = component_manager_get_component_context("TestComponent", &handle);
  auto p = std::unique_ptr<std::remove_pointer<component_context_h>::type,
       decltype(component_context_destroy)*>(handle, component_context_destroy);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_NE(p, nullptr);
  EXPECT_NE(p.get(), nullptr);
}

TEST_F(ComponentManagerTest, component_manager_get_component_context_Negative) {
  int ret = component_manager_get_component_context(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentManagerTest, component_manager_get_component_info_Positive) {
  component_info_h handle = nullptr;
  int ret = component_manager_get_component_info("TestComponent", &handle);
  auto p = std::unique_ptr<std::remove_pointer<component_info_h>::type,
       decltype(component_info_destroy)*>(handle, component_info_destroy);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_NE(p, nullptr);
  EXPECT_NE(p.get(), nullptr);
}

TEST_F(ComponentManagerTest, component_manager_get_component_info_Negative) {
  int ret = component_manager_get_component_info(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentManagerTest, component_manager_is_running_Positive) {
  bool running = false;
  int ret = component_manager_is_running("TestComponent", &running);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
  EXPECT_EQ(running, true);
}

TEST_F(ComponentManagerTest, component_manager_is_running_Negative) {
  int ret = component_manager_is_running(nullptr, nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentManagerTest, component_manager_resume_component_Positive) {
  component_context_h handle = GetHandle();

  int ret = component_manager_resume_component(handle);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
}

TEST_F(ComponentManagerTest, component_manager_resume_component_Negative) {
  int ret = component_manager_resume_component(nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}

TEST_F(ComponentManagerTest,
    component_manager_terminate_bg_component_Positive) {
  component_context_h handle = GetHandle();

  int ret = component_manager_terminate_bg_component(handle);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_NONE);
}

TEST_F(ComponentManagerTest,
    component_manager_terminate_bg_component_Negative) {
  int ret = component_manager_terminate_bg_component(nullptr);
  EXPECT_EQ(ret, COMPONENT_MANAGER_ERROR_INVALID_PARAMETER);
}
